/* 
Perform blink tracking on live video image.

Orion Sky Lawlor, lawlor@alaska.edu, 2013-10-22 (Public Domain)
*/
#include <stdlib.h> /* for NULL */
#include <fstream>
#include <opencv2/opencv.hpp>
#include <iostream>  

double blink_Hz=4.0;

/* Return the current wall clock time, in seconds */
double time_in_seconds(void) {
        return cv::getTickCount()/cv::getTickFrequency();
}


int main(int argc,char *argv[])
{  

	cv::VideoCapture *cap=0;
	long framecount=0;
	
	int argi=1; // command line argument index
	while (argc>=argi+2) { /* keyword-value pairs */
		char *key=argv[argi++];
		char *value=argv[argi++];
		if (0==strcmp(key,"--cam")) cap=new cv::VideoCapture(atoi(value));
		else if (0==strcmp(key,"--file")) cap=new cv::VideoCapture(value);
		else printf("Unrecognized command line argument '%s'!\n",key);
	}
	// Initialize capturing live feed from the camera
	if (!cap) cap=new cv::VideoCapture(0);

	// Couldn't get a device? Throw an error and quit
	if(!cap->isOpened())
	{
		printf("Could not initialize capturing...\n");
		return -1;
	}

	// Make the windows we'll be using
	cv::namedWindow("video");

	cv::Mat frame, last_frame;
	cv::Mat re,im; // real and imaginary parts of accumulator
	cv::Mat mag; // magnitude of accumulator
	cv::Mat debug; // RGB debug image
	
	
	double phase=0.0; // current blink phase angle, radians
	double last=time_in_seconds();
	// An infinite loop
	while(true)
	{
		(*cap)>>frame; // grab next frame from camera

		// If we couldn't grab a frame... quit
		if(frame.empty())
			break;
		
		framecount++;
		double cur=time_in_seconds();
		double elapsed=cur-last;
		last=cur;
		std::cout<<"Frame time: "<<(int)(elapsed*1.0e3)<<" ms\n";

		if (re.empty()) { // set image sizes first time around
			last_frame=frame.clone();
			re=cv::Mat::zeros(frame.rows,frame.cols,CV_32F);
			im=cv::Mat::zeros(frame.rows,frame.cols,CV_32F);
			mag=cv::Mat::zeros(frame.rows,frame.cols,CV_32F);
			debug=frame;
		}
		
		
		static double peak_integrator=0.0, peak_min=1000.0, peak_max=0.0;
		static int experiment_no=0;
		enum {n_experiments=101};
		enum {n_frame_count=100};
		static int frame_count=0;
		
		static std::ofstream experiment_log("experiment_log.txt");
		
		
		double period=1.0/blink_Hz; // blink period, in seconds
		double omega=2.0*M_PI*blink_Hz; // rotation rate in radians/second

		frame_count++;
		if (frame_count>2*n_frame_count) {
			experiment_log<<blink_Hz<<"	"<<peak_integrator/n_frame_count<<"	"<<peak_min<<"	"<<peak_max<<"\n";
			experiment_log.flush();
			frame_count=0; 
			peak_integrator=0.0, peak_min=1000.0, peak_max=0.0;
			experiment_no++;
			if (experiment_no>=n_experiments) exit(0);
		}
		
		
		// Advance phase by the real time difference
		//   This should be equivalent to phase=fmod(cur*omega,2*pi);
		//   but this version should have better roundoff behavior.
		phase=fmod((phase+elapsed*omega),2.0*M_PI);

		double scale=1.0/255.0;
		double s=sin(phase)*scale, c=cos(phase)*scale;
		double secatten=2.0; // seconds to average over (longer: better noise rejection, slower response)
		double frameatten=secatten/elapsed;
		double atten=1.0-1.0/frameatten; // per-frame attenuation

		for (int y=0;y<re.rows;y++)
		for (int x=0;x<re.cols;x++)
		{
			float hipass=
				frame.at<cv::Vec3b>(y,x)[2] -
				last_frame.at<cv::Vec3b>(y,x)[2];
			re.at<float>(y,x) = re.at<float>(y,x)*atten + c*hipass; 
			im.at<float>(y,x) = im.at<float>(y,x)*atten + s*hipass;
		}

		cv::magnitude( re,im, mag);
		mag=mag*(5.0/frameatten)-0.01; // throw out low-signal areas

		double maxmag=0.0; double maxx=0.0, maxy=0.0;
		double summag=0.0, sumx=0.0, sumy=0.0;
		for (int y=0;y<re.rows;y++)
		for (int x=0;x<re.cols;x++) {
			float m=mag.at<float>(y,x);
			if (m<=0.0) continue;
			summag+=m;
			sumx+=m*x; sumy+=m*y;
			
			if (maxmag<m) {
				maxmag=m;
				maxx=x; maxy=y;
			}
		}
		double avgx=0.0, avgy=0.0;
		if (summag>0.0) {avgx=sumx/summag; avgy=sumy/summag;}
		printf("T:	%.3f	mag:	%.1f	%.1f	%.1f	avg_xyM	%.1f	%.1f	%.1f	max_xyM\n",
			cur,
			avgx,avgy,summag,	
			maxx,maxy,maxmag);
		fflush(stdout);
		
		if (frame_count>n_frame_count) {
			peak_integrator+=maxmag;
			peak_min=std::min(peak_min,maxmag);
			peak_max=std::max(peak_max,maxmag);
		}
		
		// rgb[3]={}; cv::merge(rgb,3,debug); 
		
		cv::imshow("video",mag);
		
		// cv::swap(last_frame,frame); // double buffer (YU no work?)
		last_frame=frame.clone(); // slower than above, but works!
		
		// Wait for a keypress (for up to 1ms)
		int keyboardKey = cv::waitKey(1);
		if(keyboardKey!=-1)
		{
			// If any key is pressed, break out of the loop
			break;
		}
	}

	return 0;
} 


 

